package com.maileon.api.blacklists;

import com.maileon.api.XmlUtils;
import com.maileon.api.utils.MimeTypes;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Provider
@Consumes(MimeTypes.MAILEON_API_XML)
@Produces(MimeTypes.MAILEON_API_XML)
public class BlacklistsXmlSerializer implements MessageBodyReader<List<Blacklist>>, MessageBodyWriter<List<Blacklist>> {

    private static final String BLACKLISTS_ELEMENT_NAME = "blacklists";

    @Override
    public boolean isReadable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return List.class.isAssignableFrom(type);
    }

    @Override
    public List<Blacklist> readFrom(Class<List<Blacklist>> type, Type type1, Annotation[] antns,
            MediaType mt, MultivaluedMap<String, String> mm, InputStream in) throws IOException,
            WebApplicationException {
        try {
            Element filters = XmlUtils.parseXmlDocument(in);
            return deserialize(filters);
        } catch (Throwable e) {
            throw new WebApplicationException(e);
        }
    }

    public static List<Blacklist> deserialize(Element blacklists) {
        List<Blacklist> result = new ArrayList<>();
        if (BLACKLISTS_ELEMENT_NAME.equals(blacklists.getName())) {
            for (@SuppressWarnings("rawtypes")
            Iterator iterator = blacklists.elementIterator(); iterator.hasNext();) {
                Element blacklistElement = (Element) iterator.next();
                result.add(BlacklistXmlSerializer.deserialize(blacklistElement));
            }
        }
        return result;
    }

    @Override
    public boolean isWriteable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return List.class.isAssignableFrom(type);
    }

    @Override
    public void writeTo(List<Blacklist> t, Class<?> type, Type type1, Annotation[] antns, MediaType mt,
            MultivaluedMap<String, Object> mm, OutputStream out) throws IOException, WebApplicationException {
        if (t == null) {
            throw new WebApplicationException();
        }

        Element root;
        try {
            root = serialize(t);
            XMLWriter writer = new XMLWriter(out, OutputFormat.createPrettyPrint());
            writer.write(root);
            writer.close();
        } catch (Exception e) {
            throw new WebApplicationException(e);
        }

    }

    public static Element serialize(List<Blacklist> blacklists) {
        Element blacklistsElement = DocumentHelper.createElement(BLACKLISTS_ELEMENT_NAME);
        for (Blacklist blacklist : blacklists) {
            blacklistsElement.add(BlacklistXmlSerializer.serialize(blacklist));
        }
        return blacklistsElement;
    }

    @Override
    public long getSize(List<Blacklist> t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return -1;
    }

}
