package com.maileon.api.targetgroups.xml;

import com.maileon.api.XmlUtils;
import com.maileon.api.contactfilters.ContactFilter;
import com.maileon.api.targetgroups.TargetGroup;
import org.dom4j.Element;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import java.io.IOException;
import java.io.InputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;


/**
 *	De-/serializes a single contact filter
 *
 */
public class TargetGroupXmlSerializer implements MessageBodyReader<TargetGroup> {


    public static final String XML_TARGET_GROUP = "targetgroup";

    public static final String XML_ID = "id";
    public static final String XML_NAME = "name";
    public static final String XML_AUTHOR = "author";
    public static final String XML_STATE = "state";
    public static final String XML_TYPE = "type";
    public static final String XML_CONTACT_FILTER_NAME = "contact_filter_name";
    public static final String XML_CONTACT_FILTER_ID = "contact_filter_id";
    public static final String XML_EVALUATED = "evaluated";
    public static final String XML_CREATED = "created";
    public static final String XML_UPDATED = "updated";
    public static final String XML_COUNT_ACTIVE_CONTACTS = "count_active_contacts";
    public static final String XML_COUNT_CONTACTS = "count_contacts";

    @Override
    public boolean isReadable(Class<?> type, Type type1, Annotation[] antns, MediaType mt) {
        return type1 == ContactFilter.class;
    }

    @Override
    public TargetGroup readFrom(Class<TargetGroup> type, Type type1, Annotation[] antns, MediaType mt, MultivaluedMap<String, String> mm, InputStream in) throws IOException, WebApplicationException {
        try {
            Element filter = XmlUtils.parseXmlDocument(in);
            return deserializeSingleElement(filter);
        } catch (Throwable e) {
            throw new WebApplicationException(e);
        }
    }

    /**
     * Deserialize a single target group
     *
     * @param targetGroup
     * @return
     * @throws Exception
     */
    public static TargetGroup deserializeSingleElement(Element targetGroup) throws Exception {

        if (XML_TARGET_GROUP.equals(targetGroup.getName())) {
            TargetGroup cf = new TargetGroup();

            // ID
            Element id = targetGroup.element(XML_ID);
            if (id != null) {
                try {
                    cf.setId(Long.parseLong(id.getText()));
                } catch (Exception e) {
                    throw new NumberFormatException("Not a valid ID: " + id.getText());
                }
            }

            // Name
            Element name = targetGroup.element(XML_NAME);
            if (name != null) {
                cf.setName(name.getText());
            }

            // Author
            Element author = targetGroup.element(XML_AUTHOR);
            if (author != null) {
                cf.setAuthor(author.getText());
            }

            // State
            Element state = targetGroup.element(XML_STATE);
            if (state != null) {
                cf.setState(state.getText());
            }

            // Type
            Element type = targetGroup.element(XML_TYPE);
            if (type != null) {
                cf.setType(type.getText());
            }

            // Contact filter name
            Element cfn = targetGroup.element(XML_CONTACT_FILTER_NAME);
            if (cfn != null) {
                cf.setContactFilterName(cfn.getText());
            }

            // Contact filter ID
            Element cfid = targetGroup.element(XML_CONTACT_FILTER_ID);
            if (cfid != null) {
                try {
                    cf.setContactFilterId(Long.parseLong(cfid.getText()));
                } catch (Exception e) {
                    throw new NumberFormatException("Not a valid ID (long): " + cfid.getText());
                }
            }

            // Created
            Element evaluated = targetGroup.element(XML_EVALUATED);
            if (evaluated != null) {
                try {
                    cf.setEvaluated(evaluated.getText());
                } catch (Exception e) {
                    throw new NumberFormatException("Not a valid date: " + evaluated);
                }
            }
            // Created
            Element created = targetGroup.element(XML_CREATED);
            if (created != null) {
                try {
                    cf.setCreated(created.getText());
                } catch (Exception e) {
                    throw new NumberFormatException("Not a valid date: " + created);
                }
            }
            // Created
            Element updated = targetGroup.element(XML_UPDATED);
            if (updated != null) {
                try {
                    cf.setUpdated(updated.getText());
                } catch (Exception e) {
                    throw new NumberFormatException("Not a valid date: " + updated);
                }
            }

            // Count active contacts
            Element count_active_contacts = targetGroup.element(XML_COUNT_ACTIVE_CONTACTS);
            if (count_active_contacts != null) {
                try {
                    cf.setCountActiveContacts(Integer.parseInt(count_active_contacts.getText()));
                } catch (Exception e) {
                    throw new NumberFormatException("Not a int number for counted active contacts: " + count_active_contacts.getText());
                }
            }

            // Count contacts
            Element count_contacts = targetGroup.element(XML_COUNT_CONTACTS);
            if (count_contacts != null) {
                try {
                    cf.setCountContacts(Integer.parseInt(count_contacts.getText()));
                } catch (Exception e) {
                    throw new NumberFormatException("Not a int number for counted contacts: " + count_contacts.getText());
                }
            }

            return cf;

        } else throw new Exception(String.format("Root element of a single contact filter must be named '%s' but is named: '%s'", XML_TARGET_GROUP, targetGroup.getName()));
    }
}
