package com.maileon.api.transactions.serializer;

import com.maileon.api.transactions.ContactReference;
import com.maileon.api.transactions.ImportContactReference;
import com.maileon.api.transactions.ImportReference;
import org.apache.commons.io.IOUtils;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;

@Provider
@Produces(MediaType.APPLICATION_JSON)
public class ImportReferenceJsonSerializer implements MessageBodyReader<ImportReference>, MessageBodyWriter<ImportReference> {

    @Override
    public boolean isReadable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return type == ImportReference.class && mediaType.getSubtype().endsWith("json");
    }

    @Override
    public ImportReference readFrom(Class<ImportReference> type, Type genericType, Annotation[] annotations,
            MediaType mediaType, MultivaluedMap<String, String> httpHeaders, InputStream entityStream)
            throws IOException, WebApplicationException {
        try {
            JSONObject object = (JSONObject) new JSONParser().parse(new InputStreamReader(entityStream, StandardCharsets.UTF_8));
            return deserialize(object);
        } catch (ParseException e) {
            throw new WebApplicationException(e);
        }
    }

    public static ImportReference deserialize(JSONObject contactObject) {
        ImportReference result = new ImportReference();
        ImportContactReference contact = ImportContactReferenceJsonSerializer.deserialize((JSONObject) contactObject.get("contact"));
        result.setContact(contact);
        return result;
    }

    @Override
    public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return type == ContactReference.class && mediaType.getSubtype().endsWith("json");
    }

    @Override
    public void writeTo(ImportReference t, Class<?> type, Type genericType, Annotation[] annotations,
            MediaType mediaType, MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream)
            throws IOException, WebApplicationException {
        IOUtils.write(serialize(t).toJSONString(), entityStream, StandardCharsets.UTF_8);
    }

    public static JSONObject serialize(ImportReference reference) {
        JSONObject result = new JSONObject();
        if (reference != null) {
            maybePut("contact", ImportContactReferenceJsonSerializer.serialize(reference.getContact()), result);
        }
        return result;
    }

    @SuppressWarnings("unchecked")
    private static <T> void maybePut(String key, T value, JSONObject object) {
        if (value != null) {
            object.put(key, value);
        }
    }

    @Override
    public long getSize(ImportReference t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
        return -1;
    }

}
