package com.maileon.api.utils;

import com.maileon.api.Page;
import com.maileon.api.ResponseWrapper;

/**
 * This helper class manages {@link Page} creation and management
 *
 */
public class PageUtils {


	/**
	 * This method generates a page object with the given parameters.
	 * @param pageIndex The index of the page (starting from 1)
	 * @param pageSize The size of the page (elements on each page)
	 * @param totalItems The total number of elements available
	 * @param countPages The number of pages available
	 * @return a Page
	 */
	public static <T> Page<T> createPage(int pageIndex, int pageSize, int totalItems, int countPages) {
		Page<T> page = new Page<>();
		page.setPageIndex((pageIndex<=countPages)?pageIndex:countPages);
		page.setPageSize(pageSize);
		page.setTotalItems(totalItems);
		page.setCountPages(countPages);
		return page;
	}




	/**
	 * This method generates a page object with the given parameters from a response object.
	 * @param pageIndex The index of the page (starting from 1)
	 * @param pageSize The size of the page (elements on each page)
	 * @param response The response from the webservice that is parsed for X-Items and X-Pages to generate the new page
	 * @return a Page
	 */
	public static <T> Page<T> createPage(int pageIndex, int pageSize, ResponseWrapper response) {
		return createPage(pageIndex, pageSize, Integer.parseInt(response.getHeaders().getFirst("X-Items")), Integer.parseInt(response.getHeaders().getFirst("X-Pages")));
	}
}
